package javax.management;

import java.lang.reflect.Proxy;
import com.sun.jmx.mbeanserver.Introspector;
import java.lang.reflect.Modifier;

public class JMX
{
    static final JMX proof;
    public static final String DEFAULT_VALUE_FIELD = "defaultValue";
    public static final String IMMUTABLE_INFO_FIELD = "immutableInfo";
    public static final String INTERFACE_CLASS_NAME_FIELD = "interfaceClassName";
    public static final String LEGAL_VALUES_FIELD = "legalValues";
    public static final String MAX_VALUE_FIELD = "maxValue";
    public static final String MIN_VALUE_FIELD = "minValue";
    public static final String MXBEAN_FIELD = "mxbean";
    public static final String OPEN_TYPE_FIELD = "openType";
    public static final String ORIGINAL_TYPE_FIELD = "originalType";
    
    private JMX() {
    }
    
    public static <T> T newMBeanProxy(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz) {
        return newMBeanProxy(mBeanServerConnection, objectName, clazz, false);
    }
    
    public static <T> T newMBeanProxy(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz, boolean b) {
        return createProxy(mBeanServerConnection, objectName, clazz, b, false);
    }
    
    public static <T> T newMXBeanProxy(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz) {
        return newMXBeanProxy(mBeanServerConnection, objectName, clazz, false);
    }
    
    public static <T> T newMXBeanProxy(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz, boolean b) {
        return createProxy(mBeanServerConnection, objectName, clazz, b, true);
    }
    
    public static boolean isMXBeanInterface(Class<?> clazz) {
        if (!clazz.isInterface()) {
            return false;
        }
        if (!Modifier.isPublic(clazz.getModifiers()) && !Introspector.ALLOW_NONPUBLIC_MBEAN) {
            return false;
        }
        MXBean mxBean = (MXBean)clazz.getAnnotation(MXBean.class);
        if (mxBean != null) {
            return mxBean.value();
        }
        return clazz.getName().endsWith("MXBean");
    }
    
    private static <T> T createProxy(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz, boolean b, boolean b2) {
        try {
            if (b2) {
                Introspector.testComplianceMXBeanInterface(clazz);
            }
            else {
                Introspector.testComplianceMBeanInterface(clazz);
            }
        }
        catch (NotCompliantMBeanException cause) {
            throw new IllegalArgumentException(cause);
        }
        MBeanServerInvocationHandler h = new MBeanServerInvocationHandler(mBeanServerConnection, objectName, b2);
        Class[] interfaces;
        if (b) {
            interfaces = new Class[] { clazz, NotificationEmitter.class };
        }
        else {
            interfaces = new Class[] { clazz };
        }
        return clazz.cast(Proxy.newProxyInstance(clazz.getClassLoader(), interfaces, h));
    }
    
    static {
        proof = new JMX();
    }
}
